/* -*- ObjC -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the iKtai Library.

   The iKtai Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The iKtai Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the iKtai Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTestResult.m,v 1.3 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#include <CocoUnit/CocoTestResult.h>
#include <CocoUnit/CocoTestFailure.h>
#include <CocoUnit/CocoTest.h>
#include <CocoUnit/CocoTestCase.h>
#include <CocoUnit/CocoProtectable.h>
#include <CocoUnit/CocoTestListener.h>
#include <CocoUnit/CocoAssertionFailedError.h>
#include <CocoUnit/CocoError.h>
#include <Foundation/NSError.h>

@implementation CocoTestResult

+(id) testResult
{
  return [ [ [ self alloc ] init ] autorelease ];
}

-(id) init
{
  [ super init ];
  failures = [ [ NSMutableArray alloc ] init ];
  errors = [ [ NSMutableArray alloc ] init ];
  listeners = [ [ NSMutableArray alloc ] init ];
  runTests = 0;
  stop = false;
  return self;
}

-(void)dealloc
{
  [ failures release ];
  [ errors release ];
  [ listeners release ];
  [ super dealloc ];
}

-(void) addTest:(CocoTest*) test error:(NSException*) error
{
  [ errors addObject: [ CocoTestFailure failureWithTest: test
					exception: error ] ];
  NSEnumerator* e = [ listeners objectEnumerator ];
  id listener;
  while (listener = [ e nextObject ])
    {
      [ listener addTest: test error: error ];
    }
}

-(void) addTest:(CocoTest*) test failure:(CocoAssertionFailedError*) failure
{
  [ failures addObject: [ CocoTestFailure failureWithTest: test
					  exception: failure ] ];
  NSEnumerator* e = [ listeners objectEnumerator ];
  id listener;
  while (listener = [ e nextObject ])
    {
      [ listener addTest: test failure: failure ];
    }
}

-(void) addListener:(CocoTestListener*) aListener
{
  [ listeners addObject: aListener ];
}

-(void) removeListener:(CocoTestListener*) aListener
{
  [ listeners removeObject: aListener ];
}

-(void) endTest:(CocoTest*) test
{
  NSEnumerator* enumerator = [ listeners objectEnumerator ];
  CocoTestListener* listener;
  while ((listener = [ enumerator nextObject ]) != nil)
    {
      [ listener endTest: test ];
    }
}

-(unsigned int) errorCount
{
  return [ errors count ];
}

-(NSEnumerator*) errors
{
  return [ errors objectEnumerator ];
}

-(unsigned int) failureCount
{
  return [ failures count ];
}

-(NSEnumerator*) failures
{
  return [ failures objectEnumerator ];
}

-(void) runWithTest:(CocoTestCase*) test
{
  [ self startTest: test ];
  CocoProtectable* p = [ [ CocoProtectable alloc ] initWithTest: test ];
  [ self runProtected: p ];
  [ p release ];
  [ self endTest: test ];
}

-(int) runCount
{
  return runTests;
}

-(void) runProtected:(CocoProtectable*) p
{
  @try
  {
    [ p protect ];
  }
  @catch (CocoAssertionFailedError* f)
  {
    [ self addTest: p->test failure: f ];
  }
  @catch (NSException* e)
  {
    [ self addTest: p->test error: e ];
  }
  @catch (NSError* ne)
  {
    CocoError* e = [ CocoError error: [ NSString stringWithFormat: @"%@(%d)",
						 [ ne domain ], [ ne code ] ] ];
    [ self addTest: p->test error: e ];
  }
}

-(bool) shouldStop
{
  return stop;
}

-(void) startTest:(CocoTest*) test
{
  const int count = [ test countTestCases ];
  runTests += count;
  NSEnumerator* enumerator = [ listeners objectEnumerator ];
  id listener;
  while (listener = [ enumerator nextObject ])
    {
      [ listener startTest: test ];
    }
}

-(void) stop
{
  stop = true;
}

-(bool) wasSuccessful
{
  return [ self failureCount ] == 0 && [ self errorCount ] == 0;
}

@end
