/* -*- ObjC -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Library.

   The CocoUnit Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTestSuite.h,v 1.2 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#ifndef COCOTESTSUITE_H
#define COCOTESTSUITE_H

#include <CocoUnit/CocoTest.h>
#include <objc/objc-class.h>
#include <objc/objc-runtime.h>

/**
 * A <code>TestSuite</code> is a <code>Composite</code> of Tests.
 * It runs a collection of test cases. Here is an example using
 * the dynamic test definition.
 * <pre>
 * TestSuite suite= new TestSuite();
 * suite.addTest(new MathTest("testAdd"));
 * suite.addTest(new MathTest("testDivideByZero"));
 * </pre>
 * Alternatively, a TestSuite can extract the tests to be run automatically.
 * To do so you pass the class of your TestCase class to the
 * TestSuite constructor.
 * <pre>
 * TestSuite suite= new TestSuite(MathTest.class);
 * </pre>
 * This constructor creates a suite with all the methods
 * starting with "test" that take no arguments.
 *
 * @see Test
 */
@interface CocoTestSuite : CocoTest {
  NSMutableArray* tests;
  NSString* name;
}

/**
 * Constructs a TestSuite.
 */
+(id) testSuite;

/**
 * Constructs a TestSuite from the given class. Adds all the methods
 * starting with "test" as test cases to the suite.
 */
+(id) testSuiteWithClass:(Class) testClass;

/**
 * Constructs an empty TestSuite.
 */
+(id) testSuiteWithName:(NSString*) suiteName;

/**
 * Constructs a TestSuite.
 */
-(id) init;

/**
 * Constructs a TestSuite from the given class. Adds all the methods
 * starting with "test" as test cases to the suite.
 */
-(id) initWithClass:(Class) testClass;

/**
 * Constructs an empty TestSuite.
 */
-(id) initWithName:(NSString*) suiteName;

/**
 * Destructs a TestSuite.
 */
-(void) dealloc;

/**
 * Adds a test to the suite.
 */
-(void) addTest:(CocoTest*) test;

/**
 * Adds a test to the suite.
 */
-(void) addTestSuite:(Class) testClass;

/**
 * Adds a test to the suite.
 */
-(void) addTestMethod:(Method) method
		names:(NSMutableArray*) names
		class:(Class) class;

/**
 * 
 */
-(bool) isTestMethod:(Method) method;

/**
 * Counts the number of test cases that will be run by this test.
 */
-(int) countTestCases;

/**
 * Runs the tests and collects their result in a TestResult.
 */
-(void) runWithResult:(CocoTestResult*) result;

/**
 * Returns the test at the given index 
 */
-(CocoTest*) testAt:(unsigned int) atIndex;

/**
 * Returns the number of tests in this suite 
 */
-(unsigned int) testCount;

/**
 * Returns the tests as an enumeration
 */
-(NSEnumerator*) tests;

/**
 * Returns a test which will fail and log a warning message.
 */
-(CocoTest*) warning:(NSString*) message;

/**
 * Returns an NSString that represents the contents of the receiving
 * class.
 */
-(NSString*) description;

@end

#endif // COCOTESTSUITE_H
