/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: TestCaseTest.m,v 1.4 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 * 
 */

#include "TestCaseTest.h"
#include "../WasRun.h"
#include "NoArgTestCaseTest.h"
#include <CocoUnit/CocoError.h>
#include <CocoUnit/CocoTestResult.h>
#include <CocoUnit/CocoTestFailure.h>
#include <CocoUnit/CocoTestSuite.h>

@interface Error : CocoTestCase {} @end
@implementation Error
-(void) runTest
{
  [ CocoError raise ];
}
@end

@interface SetupFails : CocoTestCase {} @end
@implementation SetupFails
-(void) setUp
{
  [ CocoError raise ];
}
-(void) runTest
{
}
@end

@interface Success : CocoTestCase {} @end
@implementation Success
-(void) runTest
{
}
@end

@interface Failure : CocoTestCase {} @end
@implementation Failure
-(void) runTest
{
  [ self fail ];
}
@end

@interface TearDownFails : CocoTestCase {} @end
@implementation TearDownFails
-(void) tearDown
{
  [ self error ];
}
-(void) runTest
{
}
@end

@interface NamelessTestCase : CocoTestCase {} @end
@implementation NamelessTestCase
@end

@interface TornDown : CocoTestCase
{
 @public
  bool fTornDown;
}
@end
@implementation TornDown
-(id) initWithName:(NSString*) className
{
  [ super initWithName: className ];
  fTornDown = false;
  return self;
}
-(void) setUp
{
}
-(void) tearDown
{
  fTornDown = true;
}
-(void) runTest
{
  [ self error ];
}
@end

@interface RunAndTearDownFails : TornDown {} @end
@implementation RunAndTearDownFails
-(void) tearDown
{
  [ super tearDown ];
  [ CocoError raise ];
}
-(void) runTest
{
  [ CocoError raise ];
}
@end

@interface ExceptionRunningAndTearDown : TornDown {} @end
@implementation ExceptionRunningAndTearDown
-(void) tearDown
{
  [ self error: @"tearDown" ];
}
@end

@interface TearDownSetupFails : TornDown {} @end
@implementation TearDownSetupFails
-(void) setUp
{
  [ self error ];
}
@end

@implementation TestCaseTest

-(void) verifyError:(CocoTestCase*) test
{
  CocoTestResult* result = [ test run ];
  [ self assertEqualsLong: 1 actual: [ result runCount ] ];
  [ self assertEqualsLong: 0 actual: [ result failureCount ] ];
  [ self assertEqualsLong: 1 actual: [ result errorCount ] ];
}

-(void) verifyFailure:(CocoTestCase*) test
{
  CocoTestResult* result = [ test run ];
  [ self assertEqualsLong: 1 actual: [ result runCount ] ];
  [ self assertEqualsLong: 1 actual: [ result failureCount ] ];
  [ self assertEqualsLong: 0 actual: [ result errorCount ] ];
}

-(void) verifySuccess:(CocoTestCase*) test
{
  CocoTestResult* result = [ test run ];
  [ self assertEqualsLong: 1 actual: [ result runCount ] ];
  [ self assertEqualsLong: 0 actual: [ result failureCount ] ];
  [ self assertEqualsLong: 0 actual: [ result errorCount ] ];
}

-(void) testCaseToString
{
  NSString* str = [ NSString stringWithString: @"testCaseToString(TestCaseTest)" ];
  [ self assertEquals: str actual: [ self description ] ];
}

-(void) testError
{
  id error = [ Error testCaseWithName: @"error" ];
  [ self verifyError: error ];
}

-(void) testRunAndTearDownFails
{
  TornDown* fails = [ RunAndTearDownFails testCase ];
  [ self verifyError: fails ];
  [ self assertTrue: fails->fTornDown ];
}

-(void) testSetupFails
{
  id fails = [ SetupFails testCaseWithName: @"success" ];
  [ self verifyError: fails ];
}

-(void) testSuccess
{
  id success = [ Success testCaseWithName: @"success" ];
  [ self verifySuccess: success ];
}

-(void) testFailure
{
  id failure = [ Failure testCaseWithName: @"failure" ];
  [ self verifyFailure: failure ];
}

-(void) testTearDownAfterError
{
  TornDown* fails = [ TornDown testCase ];
  [ self verifyError: fails ];
  [ self assertTrue: fails->fTornDown ];
}

-(void) testTearDownFails
{
  TornDown* fails = [ TearDownFails testCaseWithName: @"success" ];
  [ self verifyError: fails ];
}

-(void) testTearDownSetupFails
{
  TornDown* fails = [ TearDownSetupFails testCase ];
  [ self verifyError: fails ];
  [ self assertFalse: fails->fTornDown ];
}

-(void) testWasRun
{
  WasRun* test = [ WasRun testCase ];
  [ test run ];
  [ self assertTrue: test->fWasRun ];
}

-(void) testExceptionRunningAndTearDown
{
  CocoTest* t = [ ExceptionRunningAndTearDown testCase ];
  CocoTestResult* result = [ CocoTestResult testResult ];
  [ t runWithResult: result ];
  CocoTestFailure* failure = [ [ result errors ] nextObject ];
  [ self assertEquals: @"tearDown" actual: [ [ failure thrownException ] reason ] ];
}

-(void) testNoArgTestCasePasses
{
  CocoTest* t = [ CocoTestSuite testSuiteWithClass: [ NoArgTestCaseTest class ] ];
  CocoTestResult* result = [ CocoTestResult testResult ];
  [ t runWithResult: result ];
  [ self assertEqualsLong: 1 actual: [ result runCount ] ];
  [ self assertEqualsLong: 0 actual: [ result failureCount ] ];
  [ self assertEqualsLong: 0 actual: [ result errorCount ] ];
}

-(void) testNamelessTestCase
{
  CocoTestCase* t = [ NamelessTestCase testCase ];
  @try
  {
    [ t run ];
    [ self fail ];
  }
  @catch (CocoAssertionFailedError* e)
  {
  }
}

@end

