/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTextUiResultPrinter.m,v 1.3 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 * 
 */

#include "CocoTextUiResultPrinter.h"
#include <CocoUnit/CocoTestResult.h>
#include <CocoUnit/CocoTestFailure.h>
#include <CocoUnit/CocoStreamWriter.h>
#import <Foundation/Foundation.h>

@implementation CocoTextUiResultPrinter

+(id) resultPrinter:(CocoWriter*) resultWriter
{
  return [ [ [ self alloc ] initWithWriter: resultWriter ] autorelease ];
}

-(id) init
{
  id aWriter = [ [ CocoStreamWriter alloc ] init ];
  id aRunner = [ self initWithWriter: aWriter ];
  [ aWriter release ];
  return aRunner;
}

-(id) initWithWriter:(CocoWriter*) resultWriter
{
  [ super init ];
  writer = [ resultWriter retain ];
  column = 0;
  runTests = 0;
  verbose = false;
  return self;
}

-(void) dealloc
{
  [ writer release ];
  [ super dealloc ];
}

-(void) print:(CocoTestResult*) result runTime:(struct timeval*) runTime
{
  [ self printHeader: runTime ];
  [ self printErrors: result ];
  [ self printFailures: result ];
  [ self printFooter: result ];
}

-(void) printHeader:(struct timeval*) runTime
{
  [ [ self writer ] printFormat: @"\n" ];
  [ [ self writer ] printFormat: @"Time: %@\n",
		       [ self elapsedTimeAsString: runTime ] ];
}

-(void) printErrors:(CocoTestResult*) result
{
  [ self printDefects: [ result errors ] count: [ result errorCount ]
	 type: @"error" ];
}

-(void) printFailures:(CocoTestResult*) result
{
  [ self printDefects: [ result failures ] count: [ result failureCount ]
	 type: @"failure" ];
}

-(void) printDefects:(NSEnumerator*) booBoos count:(int) count type:(NSString*) type
{
  if (count == 0)
    return;
  if (count == 1)
    [ [ self writer ] printFormat: @"There was %d %@:\n", count, type ];
  else
    [ [ self writer ] printFormat: @"There were %d %@s:\n", count, type ];
  CocoTestFailure* failure;
  int i;
  for (i = 1; (failure = [ booBoos nextObject ]) != nil; i ++)
    {
      [ self printDefect: failure count: i ];
    }
}

-(void) printDefect:(CocoTestFailure*) booBoo count:(int) count
{
  // only public for testing purposes
  [ self printDefectHeader: booBoo count: count ];
  // [ self printDefectTrace: booBoo ];
}

-(void) printDefectHeader:(CocoTestFailure*) booBoo count:(int) count
{
  [ [ self writer ] printFormat: @"%d) %@\n", count, booBoo ];
}

-(void) printFooter:(CocoTestResult*) result
{
  if ([ result wasSuccessful ])
    {
      [ [ self writer ] printFormat: @"\n" ];
      [ [ self writer ] printFormat: @"OK" ];
      if ([ result runCount ] == 1)
	[ [ self writer ] printFormat: @" (%d test)\n", [ result runCount ] ];
      else
	[ [ self writer ] printFormat: @" (%d tests)\n", [ result runCount ] ];
    }
  else
    {
      [ [ self writer ] printFormat: @"\n" ];
      [ [ self writer ] printFormat: @"FAILURES!!!\n" ];
      [ [ self writer ] printFormat:
			     @"Tests run: %d, Failures: %d, Errors: %d\n",
			   [ result runCount ],
			   [ result failureCount ],
			   [ result errorCount ] ];
    }
  [ [ self writer ] printFormat: @"\n" ];
}

-(NSString*) elapsedTimeAsString:(struct timeval*) runTime
{
  return [ NSString stringWithFormat:@"%d.%06d", runTime->tv_sec, runTime->tv_usec];
}

-(CocoWriter*) writer
{
  return writer;
}

-(void) addTest:(CocoTest*) test error:(NSException*) t
{
  [ [ self writer ] printFormat: @"E" ];
  if (verbose)
    [ [ self writer ] printFormat: @": %@", !t ? @"" : [ t description ] ];
}

-(void) addTest:(CocoTest*) test failure:(CocoAssertionFailedError*) failure
{
  [ [ self writer ] printFormat: @"F" ];
  if (verbose)
    [ [ self writer ] printFormat: @": %@", !failure ? (id)@"" : (id)failure ];
}

-(void) endTest:(CocoTest*) test
{
  if (verbose)
    [ [ self writer ] printFormat: @".\n" ];
}

-(void) startTest:(CocoTest*) test
{
  if (verbose)
    {
      runTests ++;
      [ [ self writer ] printFormat: @"%d) %@: ", runTests, test ];
    }
  else
    {
      [ [ self writer ] printFormat: @"." ];
      if (++ column >= 40)
	{
	  [ [ self writer ] printFormat: @"\n" ];
	  column = 0;
	}
    }
}

-(void) verbose: (bool) isEnable
{
  verbose = isEnable;
}
@end
